﻿using CMS;
using CMS.Search.Azure;
using Microsoft.Azure.Search.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

[assembly: RegisterModule(typeof(CustomAzureSearrch.CustomAzureSearchModule))]

namespace CustomAzureSearrch
{
    public class CustomAzureSearchModule : CMS.DataEngine.Module
    {
        // Module class constructor, inherits from the base constructor with the code name of the module as the parameter
        public CustomAzureSearchModule()
            : base("CustomAzureSearrch.CustomAzureSearchModule")
        {
        }

        // Initializes the module. Called when the application starts.
        protected override void OnInit()
        {
            base.OnInit();
            SearchServiceManager.CreatingOrUpdatingIndex.Execute += CreatingOrUpdatingIndex_Execute;   
        }


        private void CreatingOrUpdatingIndex_Execute(object sender, CreateOrUpdateIndexEventArgs e)
        {
            Microsoft.Azure.Search.Models.Index index = e.Index;

            // Ends the handler method if the index name is not 'dg-store'
            if (!index.Name.Equals("dg-store", StringComparison.InvariantCultureIgnoreCase))
            {
                return;
            }

            // Create scoring profiles
            List<ScoringProfile> lstScoringProfiles = new List<ScoringProfile>();

            // Create weight based scoring profiles
            // SKU Name
            Dictionary<string, double> dictSKUName = new Dictionary<string, double>();
            dictSKUName.Add("skuname", 3);
            TextWeights twskuname = new TextWeights(dictSKUName);

            lstScoringProfiles.Add(new ScoringProfile()
            {
                Name = "skuname",
                TextWeights = twskuname
            });

            // Short Description
            Dictionary<string, double> dictSKUShortDescription = new Dictionary<string, double>();
            dictSKUShortDescription.Add("skushortdescription", 3);
            TextWeights twskushortdescription = new TextWeights(dictSKUShortDescription);
            lstScoringProfiles.Add(new ScoringProfile()
            {
                Name = "skushortdescription",
                TextWeights = twskushortdescription
            });

            // Coffee Country
            Dictionary<string, double> dictCoffeeCountry = new Dictionary<string, double>();
            dictCoffeeCountry.Add("coffeecountry", 3);
            TextWeights twcoffeecountry = new TextWeights(dictCoffeeCountry);
            lstScoringProfiles.Add(new ScoringProfile()
            {
                Name = "coffeecountry",
                TextWeights = twcoffeecountry
            });

            // Coffee Farm
            Dictionary<string, double> dictCoffeeFarm = new Dictionary<string, double>();
            dictCoffeeFarm.Add("coffeefarm", 3);
            TextWeights twcoffeefarm = new TextWeights(dictCoffeeFarm);
            lstScoringProfiles.Add(new ScoringProfile()
            {
                Name = "coffeefarm",
                TextWeights = twcoffeefarm
            });

            // Set the scoring profiles
            index.ScoringProfiles = lstScoringProfiles;

            // Initializes the index's list of suggesters (if it does not exist)
            if (index.Suggesters == null)
            {
                index.Suggesters = new List<Suggester>();
            }

            // Prepares a suggester object for further configuration
            Suggester suggester;

            // Checks whether the index already contains a suggester named 'productnamesuggester'
            Suggester existingSuggester = index.Suggesters.FirstOrDefault(s => s.Name == "productnamesuggester");
            if (existingSuggester != null)
            {
                // Uses the existing suggester
                suggester = existingSuggester;
            }
            else
            {
                // Adds a suggester named 'productnamesuggester' to the index
                suggester = new Suggester
                {
                    Name = "productnamesuggester",
                    SourceFields = new List<string>(),
                    SearchMode = SuggesterSearchMode.AnalyzingInfixMatching
                };
                index.Suggesters.Add(suggester);
            }

            // Creates a dictionary containing the index's fields
            Dictionary<string, Field> indexFields = index.Fields.ToDictionary(f => f.Name, StringComparer.InvariantCultureIgnoreCase);

            // Confirms that the index contains the 'skuname' field and that it is not yet added as a suggester source field
            if (indexFields.ContainsKey("skuname") && !suggester.SourceFields.Contains("skuname", StringComparer.InvariantCultureIgnoreCase))
            {
                // Adds the 'skuname' fields as a source for the content of suggestions
                suggester.SourceFields.Add("skuname");
            }
        }
    }
}
